#!/usr/local/bin/wish

# old versions of BLT (<=2.1) don't report a version number
if [catch {set blt_version}] {set blt_version 2.1}
catch {blt::graph}
catch {namespace import blt::graph}
catch {graph}

#----------------------------------------------------------------
# where are we?
set refgui(script) [info script]
# translate links -- go six levels deep
foreach i {1 2 3 4 5 6} {
    if {[file type $refgui(script)] == "link"} {
	set link [file readlink $refgui(script)]
	if { [file  pathtype  $link] == "absolute" } {
	    set refgui(script) $link
	} {
	    set refgui(script) [file dirname $refgui(script)]/$link
	}
    } else {
	break
    }
}
# fixup relative paths
if {[file pathtype $refgui(script)] == "relative"} {
    set refgui(script) [file join [pwd] $refgui(script)]
}
set refgui(scriptdir) [file dirname $refgui(script) ]
set refgui(rietveld) [file join $refgui(scriptdir) bin rietveld]
#----------------------------------------------------------------
# default values for options
set refgui(debug) 0
set opt_array(GSASBACKSPACE) 0
set opt_array(FONT) Courier
#----------------------------------------------------------------
source [file join $refgui(scriptdir) prep.tcl]
source [file join $refgui(scriptdir) refine.tcl]
if [file exists [file join ~ .refine_config]] {
    source [file join ~ .refine_config]
}

# platform-specific code
if {$tcl_platform(platform) == "windows"} {
    if [catch {package require winexec}] {
	tk_dialog .err "WINEXEC Error" \
		"Error -- Unable to load the WINEXEC package. This is needed in Win95 machines" \
	    	error 0 Quit
	destroy .
    }
    if {$tcl_platform(os) == "Windows 95"} {
	# this creates a DOS box to run a program in
	proc forknewterm {title command "input {}"} {
	    global env refgui
	    # check the path -- can DOS use it?
	    if {[string first // [pwd]] != -1} {
		tk_dialog .braindead "Invalid Path" \
		{Error -- Use "Map network drive" to access this directory with a letter (e.g. F:) \
		Win-95 can't directly access a network drive in DOS} error 0 OK
		return
	    }
	    # replace the forward slashes with backward
	    regsub -all / $command \\ command
	    # pause is hard coded in the .BAT file
	    # all winexec commands are background commands
	    if {$input != ""} {
		winexec -d [file nativename [pwd]] \
			[file join $refgui(scriptdir) bin dostcl1.bat] \
			$command $input
	    } else { 
		winexec -d [file nativename [pwd]] \
			[file join $refgui(scriptdir) bin dostcl.bat] $command
	    }
	}
    } else {
	# now for - brain-dead Windows-NT
	# this creates a DOS box to run a program in
	proc forknewterm {title command "input {}"} {
	    global env refgui
	    # all winexec commands are background commands
	    if {$input != ""} {
		winexec -d [file nativename [pwd]] cmd.exe "/c $command < $input"
	    } else { 
		winexec -d [file nativename [pwd]] cmd.exe "/c $command"
	    }
	}
    }
} else {
    # this creates a xterm window to run a program in
    proc forknewterm {title command "input {}"} {
	global env refgui opt_array
	# UNIX environment variables
	# PGPLOT_DIR is needed by PGPLOT
	set env(PGPLOT_DIR) [file join $refgui(scriptdir) bin]
	set env(PGPLOT_FONT) [file join $refgui(scriptdir) bin grfont.dat]
	set env(NIST_RIETVELD) [file join $refgui(scriptdir) bin]
	set termopts {}
	if $opt_array(GSASBACKSPACE) {
	    append termopts \
		    {-xrm "xterm*VT100.Translations: #override\\n <KeyPress>BackSpace: string(\\177)"}
	}
	append termopts " -sb"
	#
	if {$input != ""} {append command " < $input"}
	append command "\; echo -n Press Enter to continue \; read x"
	wm iconify .
	catch {eval exec xterm $termopts -title [list $title] \
		-e /bin/sh -c [list $command]} errmsg
	if $refgui(debug) {puts "xterm result = $errmsg"}
	wm deiconify .
    }
}

proc putontop {w} {
    # center window $w above its parent and make it stay on top
    set wp [winfo parent $w]
    #wm transient $w [winfo toplevel $wp]
    wm withdraw $w
    update idletasks
    # center the new window in the middle of the parent
    set x [expr [winfo x $wp] + [winfo width $wp]/2 - \
	    [winfo reqwidth $w]/2 - [winfo vrootx $wp]]
    if {$x < 0} {set x 0}
    set xborder 10
    if {$x+[winfo reqwidth $w] +$xborder > [winfo screenwidth $w]} {
	incr x [expr \
		[winfo screenwidth $w] - ($x+[winfo reqwidth $w] + $xborder)]
    }
    set y [expr [winfo y $wp] + [winfo height $wp]/2 - \
	    [winfo reqheight $w]/2 - [winfo vrooty $wp]]
    if {$y < 0} {set y 0}
    set yborder 25
    if {$y+[winfo reqheight $w] +$yborder > [winfo screenheight $w]} {
	incr y [expr \
		[winfo screenheight $w] - ($y+[winfo reqheight $w] + $yborder)]
    }
    wm geom $w +$x+$y
    wm deiconify $w

    global makenew
    set makenew(OldFocus) [focus]
    catch {
	set makenew(OldGrab) [grab current $w]
	if {$makenew(OldGrab) != ""} {
	    set makenew(GrabStatus) [grab status $makenew(OldGrab)]
	}
	grab $w
    }
}

proc afterputontop {} {
    # restore focus
    global makenew
    catch {focus $makenew(OldFocus)}
    if {$makenew(OldGrab) != ""} {
	catch {
	    if {$makenew(GrabStatus) == "global"} {
		grab -global $makenew(OldGrab)
	    } else {
		grab $makenew(OldGrab)
	    }
        }
    }
}

proc ShowBigMessage {win labeltext msg "optionlist OK"} {
    global opt_array
    catch {destroy $win}
    toplevel $win

    # grab focus, etc.
    pack [label $win.l1 -text $labeltext] -side top
    pack [frame $win.f1] -side top -expand yes -fill both
    grid [text  $win.f1.t  \
	    -height 20 -width 55  -wrap none -font $opt_array(FONT) \
	    -xscrollcommand "$win.f1.bscr set" \
	    -yscrollcommand "$win.f1.rscr set" \
	    ] -row 1 -column 0 -sticky news
    grid [scrollbar $win.f1.bscr -orient horizontal \
	    -command "$win.f1.t xview" \
	    ] -row 2 -column 0 -sticky ew
    grid [scrollbar $win.f1.rscr  -command "$win.f1.t yview" \
	    ] -row 1 -column 1 -sticky ns
    # give extra space to the text box
    grid columnconfigure $win.f1 0 -weight 1
    grid rowconfigure $win.f1 1 -weight 1
    $win.f1.t insert end $msg

    global makenew
    set makenew(result) 0
    bind $win <Return> "destroy $win"
    bind $win <KeyPress-Prior> "$win.f1.t yview scroll -1 page"
    bind $win <KeyPress-Next> "$win.f1.t yview scroll 1 page"
    bind $win <KeyPress-Right> "$win.f1.t xview scroll 1 unit"
    bind $win <KeyPress-Left> "$win.f1.t xview scroll -1 unit"
    bind $win <KeyPress-Up> "$win.f1.t yview scroll -1 unit"
    bind $win <KeyPress-Down> "$win.f1.t yview scroll 1 unit"
    bind $win <KeyPress-Home> "$win.f1.t yview 0"
    bind $win <KeyPress-End> "$win.f1.t yview end"
    set i 0
    foreach item $optionlist {
	pack [button $win.q[incr i] \
		-command "set makenew(result) $i; destroy $win" -text $item] -side left
    }
    putontop $win
    tkwait window $win

    # fix focus...
    afterputontop
    return $makenew(result)
}

proc RunRietveld {top} {
    global prep refgui
    # here we run RIETVELD
    forknewterm "Run Rietveld" "$refgui(rietveld) $prep(rootname)" 
    buttonbar $top
}

proc getfile {} {
    global prep tcl_platform
    if {$tcl_platform(platform) == "windows"} {
	set prep(rawfile) [tk_getOpenFile \
		-filetypes {{"BT-1 files" .RAW}} \
		-title "Select Data File"]
    } else {
	set prep(rawfile) [tk_getOpenFile \
		-filetypes {{"BT-1 files" .RAW} {"BT-1 files" .raw}} \
		-title "Select Data File"]
    }
}

proc LoadDataFile {} {
    global prep in tcl_platform
    if {[string trim $prep(rawfile)] == ""} return
    if {$tcl_platform(platform) != "windows"} {
	set caps [string toupper [file tail $prep(rawfile)]]
	set msg {}
	set dir [file dirname $prep(rawfile)]
	if {$caps != [file tail $prep(rawfile)]} {
	    set newname [file join $dir $caps]
	    if {[file exists $newname]} {
		set msg "Using $caps instead of [file tail $prep(rawfile)]."
	    } else {
		file rename $prep(rawfile) $newname
		set msg "Renaming file [file tail $prep(rawfile)] to $caps."
	    }
	    set prep(rawfile) $newname
	}
	if ![catch {exec grep -l \r $prep(rawfile)}] {
	    file rename -force $prep(rawfile) $prep(rawfile).unstripped
	    exec tr -d {\r} < $prep(rawfile).unstripped > $prep(rawfile)
	    if {$msg != ""} {append msg "\n"}
	    append msg "Stripping <CR> characters from [file tail $prep(rawfile)]"
	}
	if {$msg != ""} {
	    tk_dialog .warn "File changes" "$msg\n(in $dir)" warning 0 "OK"
	}
    }
    wm title . $prep(rawfile)

    set prep(dirname) [file dirname $prep(rawfile)]
    cd $prep(dirname)
    set prep(rootname) [file tail [file root $prep(rawfile)]]
    # now start initializing
    set fp [open $prep(rootname).RAW r]
    set line [gets $fp]
    # Parse the first line
    set prep(id)    [string range $line 0 3]
    set in(ID)      $prep(id)
    set prep(title) [string range $line 0 50]
    set in(title)   $prep(title)
    set prep(wave)  [lindex [string range $line 51 end] 0]
    set in(LAMBDA)  $prep(wave)
    set prep(col)   [lindex [string range $line 51 end] 1]
    set prep(mono)  [lindex [string range $line 51 end] 3]
    set line [gets $fp]
    set line [gets $fp]
    # get the step size off the third line
    close $fp
    set prep(step) [expr [lindex $line 1]/100.]
    init_prep $prep(mono) $prep(col) 
    init_refine $prep(mono) $prep(col) 
}

# called to load a prep file
proc LoadOptions {box} {
    global tcl_platform opt_array
    set top $box.options
    catch {eval destroy $top}
    grid [frame $top] -row 2 -column 0 -columnspan 8 -sticky nsw
    set i 0
    if {$tcl_platform(platform) != "windows"} {
	grid [checkbutton $top.[incr i] -text "Override backspace" \
	    -variable opt_array(GSASBACKSPACE)] -row 1 -column 1 -columnspan 2
    }
    grid [label $top.[incr i] -text "Font for output files"] -row 2 -column 1
    set option $top.[incr i]
    set menu [tk_optionMenu $option opt_array(FONT) null]
    grid $option -row 2 -column 2
    $menu delete 0 end
    foreach size {5 6 7 8 9 10 11 12 13 14 15 16} {
	$menu add radiobutton -font "Courier $size" \
		-variable opt_array(FONT) -value "Courier $size" \
		-label "Courier $size"
    }
    grid [button $top.[incr i] -text "Save settings" -command SaveOptions] \
	    -row 3 -column 1 -columnspan 2
}

proc SaveOptions {} {
    global opt_array
    set fp [open [file join ~ .refine_config] w]
    foreach item [array names opt_array] {
	puts $fp "set opt_array($item) [list $opt_array($item)]"
    }
    close $fp
}



proc buttonbar {top} {
    global prep
    if {$top == ""} {
	catch {eval destroy [grid slaves .]}
    } else {
	catch {eval destroy [grid slaves $top]}
    }
    raise $top
    catch {destroy .edit}
    set col 1
    # create an open button
    grid [button $top.$col -text Open -command "getfile; LoadDataFile"] \
	    -column $col -row 1 -sticky nw

    # create a Rietveld button; this will be used to manually run Rietveld
    incr col
    grid [button $top.$col -text Rietveld -command "RunRietveld $top"] \
	    -column $col -row 1 -sticky nw

    # create a prep button
    incr col
    grid [button $top.$col -text Prep -command "LoadPrep $top"] \
	    -column $col -row 1  -sticky nw

    # create a Refine button
    incr col
    set state disabled
    if {[file exists $prep(rootname).DAT]} {set state normal}
    grid [button $top.$col -text Refine -command "LoadRefine $top" \
	    -state $state] -column $col -row 1 -sticky nw

    # create an Options button
    incr col
    grid [button $top.$col -text Options -command "LoadOptions $top"] \
	    -column $col -row 1 -sticky nw

    # create a Quit button
    incr col
    grid [button $top.$col -text Quit -command "destroy ."] \
	    -column $col -row 1  -sticky ne
    grid columnconf $top $col -weight 1
}


set prep(rootname) {}
# center the parent window because the getExpFileName window
# will be centered above it.
wm withdraw .
set x [expr [winfo screenwidth .]/2 - [winfo reqwidth .]/2 ]
set y [expr [winfo screenheight .]/2 - [winfo reqheight .]/2]
wm geom . +$x+$y
wm deiconify .
# windows needs this update before using tk_getOpenFile. 
update
#
set prep(rawfile) {}
if {$argv != ""} {
    set prep(rawfile) [lindex $argv 0]
    if {[file extension $prep(rawfile)] != ".RAW" } {
	set prep(rawfile) $prep(rawfile).RAW
    }
}

if {![file exists $prep(rawfile)]} {
    if {$prep(rawfile) != ""} {
	tk_dialog .err "Not Found" \
		"File $prep(rawfile) does not exist" \
		warning 0 "OK"
    }
    getfile
}
LoadDataFile

if {[string trim $prep(rootname)] == ""} exit
#pack [frame .a] -side top
frame .a
grid rowconfig . 1 -weight 1 -minsize 400
grid columnconfig . 1  -weight 1 -minsize 430
grid .a -sticky new -column 1 -row 1    
buttonbar .a
